package somcolorapp;

/**
 * Package somcolorapp
 * Classification des couleurs avec un rseau de Kohonen.
 * Rseaux Neauronaux, Vuibert 2006.
 * Jean-Philippe Rennard
 * version 1.0, 17/3/2006
 */

import java.awt.*;
import java.util.*;
import neuronespack.*;

/**
 * <p>Title: Rseau Classification des couleurs</p>
 * <p>Description: Rseau de Kohonen pour la classification des couleurs.</p>
 */
public class ReseauSOMColor extends ReseauKoho {
  // Paramtres par dfaut
  public static int TAILLE_DEF = 16;//50;
  public static int MAX_ITER_DEF = 1500;//2500;
  public static int NB_COL_DEF = 6;//6;
  public static double ETA0_DEF = 0.1D;
  public static double SIGMA0_DEF = 0.9D;

  /** Pointeur sur processus */
  SOMColorProcess processSOM;
  /** Nombre de couleurs */
  int nbCouleurs;
  /** Echantillon de couleurs  traiter */
  Color[] paletteExemple;
  /** Liste des couleurs  traiter */
  static Vector listeCol;

  /**
   * Construit le rseau au sein d'un processus selon paramtres
   * @param p SOMColorProcess : processus
   * @param t int : taille d'un ct de la carte
   * @param it int : nombre maximum d'itrations
   * @param nc int : nombre de couleurs (racine cubique)
   * @param e0 double : eta0
   * @param s0 double : sigma0
   */
  public ReseauSOMColor(SOMColorProcess p,
                        int t, int it, int nc, double e0, double s0) {
    super();
    setDonneesCourantes(3, 1);
    setResultat(1);
    processSOM = p;
    setDonneesCourantes(3, t*t);
    setResultat(t*t);
    setEta0(e0);
    setSigma0(t * s0);
    construitReseau(t, t);
    boolean mc[][] = new boolean[3][t * t];
    Connexions.construitConnexionsCompletes(3, t * t, mc);
    construitConnexionsEntrees();
    setMaxIteration(it);
    setTau();
    numIteration = 0;
    int pp = nc;//6;
    nbCouleurs = (int)Math.pow(pp, 3);
    paletteExemple = new Color[nbCouleurs];
    for(int r=0; r<pp; r++) {
      for(int g=0; g<pp; g++) {
        for(int b=0; b<pp; b++) {
          paletteExemple[r*pp*pp+g*pp+b] =
              new Color(r/(float)(pp-1), g/(float)(pp-1), b/(float)(pp-1));
        }
      }
    }
    initListeCol();
  }

  /** Initialisation de la liste des couleurs  traiter */
  void initListeCol() {
    listeCol = new Vector(nbCouleurs);
    for(int i=0; i<nbCouleurs; i++) {
      listeCol.addElement(paletteExemple[i]);
    }
  }

  /** Affichage classification */
  public void affRes() {
    processSOM.somVue.afficheFond();
    double col[] = new double[3];
    for(int i=0; i<nbCouleurs; i++) {
      col[0] = paletteExemple[i].getRed()/255.0F;
      col[1] = paletteExemple[i].getGreen()/255.0F;
      col[2] = paletteExemple[i].getBlue()/255.0F;
      getDonneesCourantes().setVecteurEntree(col);
      propagation();
      processSOM.somVue.afficheCellule((int) (getBMU().getPosX()), (int)getBMU().getPosY(), paletteExemple[i] );
    }
    processSOM.somVue.repaint();
  }

  /** Apprentissage d'une couleur */
  public void epoque() {
    double col[] = new double[3];
    int r = (int)(Math.random() * listeCol.size());
    Color colCrt = (Color) listeCol.elementAt(r);
    col[0] = colCrt.getRed()/255.0F;
    col[1] = colCrt.getGreen()/255.0F;
    col[2] = colCrt.getBlue()/255.0F;
    listeCol.removeElementAt(r);
    getDonneesCourantes().setVecteurEntree(col);
    apprend();
    numIteration++;
    if(listeCol.size() == 0) {
      initListeCol();
    }
  }

  /**
   * Renvoie la couleur associe  un neurone
   * @param x int : Position du neurone en x
   * @param y int : Position du neurone en y
   * @return Color : Couleur associe aux poids des synapses
   */
  public Color getCouleurNeurone(int x, int y) {
    float red, green, blue;
    NeuroneKoho neurone = getNeuroneXY(x, y);
    red = (float) neurone.getSynapseIn(0).getPoids();
    green = (float) neurone.getSynapseIn(1).getPoids();
    blue = (float) neurone.getSynapseIn(2).getPoids();
    return new Color(red, green, blue);
  }
}
